@@ -75,18 +75,57 @@ function configure_murano_rpc_backend() {
     fi
 }
 
+function configure_murano_glare_backend() {
+    # Configure Murano to use GlARe application storage backend
+    iniset $MURANO_CONF_FILE engine packages_service 'glare'
+    if is_service_enabled murano-cfapi; then
+        iniset $MURANO_CFAPI_CONF_FILE cfapi packages_service 'glare'
+    fi
+    iniset $MURANO_CONF_FILE glare url $GLANCE_SERVICE_PROTOCOL://$GLANCE_GLARE_HOSTPORT
+    iniset $MURANO_CONF_FILE glare endpoint_type $GLARE_ENDPOINT_TYPE
+    echo -e $"\nexport MURANO_PACKAGES_SERVICE='glare'" | sudo tee -a $TOP_DIR/openrc
+    echo -e $"\nexport GLARE_URL='$GLANCE_SERVICE_PROTOCOL://$GLANCE_GLARE_HOSTPORT'" | sudo tee -a $TOP_DIR/openrc
+}
+
+function restart_glare_service() {
+    # Restart GlARe service to apply Murano artifact plugin
+    if is_running glance-glare; then
+        echo_summary "Restarting GlARe to apply config changes"
+        stop_process g-glare
+        run_process g-glare "$GLANCE_BIN_DIR/glance-glare --config-file=$GLANCE_CONF_DIR/glance-glare.conf"
+        echo "Waiting for GlARe [g-glare] ($GLANCE_GLARE_HOSTPORT) to start..."
+        if ! wait_for_service $SERVICE_TIMEOUT $GLANCE_SERVICE_PROTOCOL://$GLANCE_GLARE_HOSTPORT; then
+            die $LINENO " GlARe [g-glare] did not start"
+        fi
+    else
+        echo_summary "GlARe service wasn't started yet. It will start in usual way."
+    fi
+}
+
+function install_murano_artifact_plugin() {
+    # Provide support of Murano artifacts type to GlARe
+    setup_package $MURANO_DIR/contrib/glance -e
+}
+
+function is_murano_backend_glare() {
+    is_service_enabled g-glare && [[ "$MURANO_USE_GLARE" == "True" ]] && return 0
+    return 1
+}
+
 function configure_murano_networking {
     # Use keyword 'public' if Murano external network was not set.
     # If it was set but the network is not exist then
     # first available external network will be selected.
     local ext_net=${MURANO_EXTERNAL_NETWORK:-'public'}
-    local ext_net_id=$(neutron net-external-list \
-            | grep " $ext_net " | get_field 2)
+    local ext_net_id=$(openstack --os-cloud=devstack-admin \
+    --os-region-name="$REGION_NAME" network list \
+    --external | grep " $ext_net " | get_field 1)
 
-    # Try to select first available external network
-    if [[ -n "$ext_net_id" ]]; then
-        ext_net_id=$(neutron net-external-list -f csv -c id \
-            | tail -n +2 | tail -n 1)
+    # Try to select first available external network if ext_net_id is null
+    if [[ ! -n "$ext_net_id" ]]; then
+        ext_net_id=$(openstack --os-cloud=devstack-admin \
+        --os-region-name="$REGION_NAME" network list \
+        --external -f csv -c ID | tail -n +2 | tail -n 1)
     fi
 
     # Configure networking options for Murano
@@ -126,6 +165,14 @@ function configure_murano {
 
     iniset $MURANO_CONF_FILE DEFAULT debug $MURANO_DEBUG
     iniset $MURANO_CONF_FILE DEFAULT use_syslog $SYSLOG
+    # Format logging
+    if [ "$LOG_COLOR" == "True" ] && [ "$SYSLOG" == "False" ]; then
+        setup_colorized_logging $MURANO_CONF_FILE DEFAULT
+    else
+        # Show user_name and project_name instead of user_id and project_id
+        iniset $MURANO_CONF_FILE DEFAULT logging_context_format_string "%(asctime)s.%(msecs)03d %(levelname)s %(name)s [%(request_id)s %(user_name)s %(project_name)s] %(instance)s%(message)s"
+    fi
+
     iniset $MURANO_CONF_FILE DEFAULT home_region $REGION_NAME
 
     # Murano Policy Enforcement Configuration
@@ -160,10 +207,41 @@ function configure_murano {
     # Configure Murano API URL
     iniset $MURANO_CONF_FILE murano url "http://127.0.0.1:8082"
 
+    # Configure the number of api workers
+    if [[ -n "$MURANO_API_WORKERS" ]]; then
+        iniset $MURANO_CONF_FILE murano api_workers $MURANO_API_WORKERS
+    fi
+
     # Configure the number of engine workers
     if [[ -n "$MURANO_ENGINE_WORKERS" ]]; then
-        iniset $MURANO_CONF_FILE engine workers $MURANO_ENGINE_WORKERS
+        iniset $MURANO_CONF_FILE engine engine_workers $MURANO_ENGINE_WORKERS
     fi
+    if is_murano_backend_glare; then
+        configure_murano_glare_backend
+    fi
+}
+
+# set the murano packages service backend
+function set_packages_service_backend() {
+    if is_murano_backend_glare; then
+        MURANO_PACKAGES_SERVICE='glare'
+    else
+        MURANO_PACKAGES_SERVICE='murano'
+    fi
+}
+
+# configure_murano_cfapi() - Set config files
+function configure_murano_cfapi {
+
+    # Generate Murano configuration file and configure common parameters.
+    oslo-config-generator --namespace keystonemiddleware.auth_token \
+                          --namespace murano.cfapi \
+                          --namespace oslo.db \
+                          > $MURANO_CFAPI_CONF_FILE
+    cp $MURANO_DIR/etc/murano/murano-cfapi-paste.ini $MURANO_CONF_DIR
+
+    configure_service_broker
+
 }
 
 # install_murano_apps() - Install Murano apps from repository murano-apps, if required
@@ -175,6 +253,8 @@ function install_murano_apps() {
     # clone murano-apps only if app installation is required
     git_clone $MURANO_APPS_REPO $MURANO_APPS_DIR $MURANO_APPS_BRANCH
 
+    set_packages_service_backend
+
     # install Murano apps defined in the comma-separated list $MURANO_APPS
     for murano_app in ${MURANO_APPS//,/ }; do
         find $MURANO_APPS_DIR -type d -name "package" | while read package; do
@@ -187,6 +267,8 @@ function install_murano_apps() {
                        --os-tenant-name $OS_PROJECT_NAME \
                        --os-auth-url http://$KEYSTONE_AUTH_HOST:5000 \
                        --murano-url http://127.0.0.1:8082 \
+                       --glare-url $GLANCE_SERVICE_PROTOCOL://$GLANCE_GLARE_HOSTPORT \
+                       --murano-packages-service $MURANO_PACKAGES_SERVICE \
                        package-import \
                        --is-public \
                        --exists-action u \
@@ -200,13 +282,38 @@ function install_murano_apps() {
 
 # configure_service_broker() - set service broker specific options to config
 function configure_service_broker {
-    #Add needed options to murano.conf
-    iniset $MURANO_CONF_FILE cfapi tenant "$MURANO_CFAPI_DEFAULT_TENANT"
-    iniset $MURANO_CONF_FILE cfapi bind_host "$MURANO_SERVICE_HOST"
-    iniset $MURANO_CONF_FILE cfapi bind_port "$MURANO_CFAPI_SERVICE_PORT"
-    iniset $MURANO_CONF_FILE cfapi auth_url "http://${KEYSTONE_AUTH_HOST}:5000"
+
+    iniset $MURANO_CFAPI_CONF_FILE DEFAULT debug $MURANO_DEBUG
+    iniset $MURANO_CFAPI_CONF_FILE DEFAULT use_syslog $SYSLOG
+
+    #Add needed options to murano-cfapi.conf
+    iniset $MURANO_CFAPI_CONF_FILE cfapi tenant "$MURANO_CFAPI_DEFAULT_TENANT"
+    iniset $MURANO_CFAPI_CONF_FILE cfapi bind_host "$MURANO_SERVICE_HOST"
+    iniset $MURANO_CFAPI_CONF_FILE cfapi bind_port "$MURANO_CFAPI_SERVICE_PORT"
+    iniset $MURANO_CFAPI_CONF_FILE cfapi auth_url "http://${KEYSTONE_AUTH_HOST}:5000"
+
+    # configure the database.
+    iniset $MURANO_CFAPI_CONF_FILE database connection `database_connection_url murano_cfapi`
+
+    # Setup keystone_authtoken section
+    iniset $MURANO_CFAPI_CONF_FILE keystone_authtoken auth_uri "http://${KEYSTONE_AUTH_HOST}:5000"
+    iniset $MURANO_CFAPI_CONF_FILE keystone_authtoken auth_host $KEYSTONE_AUTH_HOST
+    iniset $MURANO_CFAPI_CONF_FILE keystone_authtoken auth_port $KEYSTONE_AUTH_PORT
+    iniset $MURANO_CFAPI_CONF_FILE keystone_authtoken auth_protocol $KEYSTONE_AUTH_PROTOCOL
+    iniset $MURANO_CFAPI_CONF_FILE keystone_authtoken cafile $KEYSTONE_SSL_CA
+    iniset $MURANO_CFAPI_CONF_FILE keystone_authtoken admin_tenant_name $SERVICE_TENANT_NAME
+    iniset $MURANO_CFAPI_CONF_FILE keystone_authtoken admin_user $MURANO_ADMIN_USER
+    iniset $MURANO_CFAPI_CONF_FILE keystone_authtoken admin_password $SERVICE_PASSWORD
+
 }
 
+function prepare_core_library() {
+    cd $MURANO_DIR/meta/io.murano && zip -r io.murano.zip .
+}
+
+function remove_core_library_zip() {
+    rm -f $MURANO_DIR/meta/io.murano/io.murano.zip
+}
 
 # init_murano() - Initialize databases, etc.
 function init_murano() {
@@ -216,10 +323,35 @@ function init_murano() {
     recreate_database murano utf8
 
     $MURANO_BIN_DIR/murano-db-manage --config-file $MURANO_CONF_FILE upgrade
-    $MURANO_BIN_DIR/murano-manage --config-file $MURANO_CONF_FILE import-package $MURANO_DIR/meta/io.murano
 }
 
+# init_murano_cfapi() - Initialize databases, etc.
+function init_murano_cfapi() {
 
+    # (re)create Murano database
+    recreate_database murano_cfapi utf8
+
+    $MURANO_BIN_DIR/murano-cfapi-db-manage --config-file $MURANO_CFAPI_CONF_FILE upgrade
+}
+
+function setup_core_library() {
+    prepare_core_library
+
+    set_packages_service_backend
+
+    murano --os-username admin \
+           --os-password $ADMIN_PASSWORD \
+           --os-tenant-name admin \
+           --os-auth-url http://$KEYSTONE_AUTH_HOST:5000 \
+           --os-region-name $REGION_NAME \
+           --murano-url http://127.0.0.1:8082 \
+           --glare-url $GLANCE_SERVICE_PROTOCOL://$GLANCE_GLARE_HOSTPORT \
+           --murano-packages-service $MURANO_PACKAGES_SERVICE \
+           package-import $MURANO_DIR/meta/io.murano/io.murano.zip \
+           --is-public
+    remove_core_library_zip
+
+}
 # install_murano() - Collect source and prepare
 function install_murano() {
     install_murano_pythonclient
@@ -227,8 +359,11 @@ function install_murano() {
     git_clone $MURANO_REPO $MURANO_DIR $MURANO_BRANCH
 
     setup_develop $MURANO_DIR
-}
 
+    if is_murano_backend_glare; then
+        install_murano_artifact_plugin
+    fi
+}
 
 function install_murano_pythonclient() {
 # For using non-released client from git branch, need to add
@@ -260,7 +395,7 @@ function stop_murano() {
 
 # start_service_broker() - start murano CF service broker
 function start_service_broker() {
-    screen_it murano-cfapi "cd $MURANO_DIR && $MURANO_BIN_DIR/murano-cfapi --config-file $MURANO_CONF_DIR/murano.conf"
+    screen_it murano-cfapi "cd $MURANO_DIR && $MURANO_BIN_DIR/murano-cfapi --config-file $MURANO_CONF_DIR/murano-cfapi.conf"
 }
 
 
@@ -293,6 +428,15 @@ function configure_murano_tempest_plugin() {
             iniset $TEMPEST_CONFIG service_available murano_cfapi "True"
             iniset $TEMPEST_CONFIG service_broker run_service_broker_tests "True"
         fi
+        if is_service_enabled g-glare; then
+            # TODO(freerunner): This is bad way to configure tempest to
+            # TODO see glare as enabled. We need to move it out to tempest
+            # TODO of glance repo when glare become official OS API.
+            iniset $TEMPEST_CONFIG service_available glare "True"
+        fi
+        if is_murano_backend_glare; then
+            iniset $TEMPEST_CONFIG application_catalog glare_backend "True"
+        fi
     fi
 }
 
@@ -330,26 +474,11 @@ MURANO_DASHBOARD_CACHE_DIR=${MURANO_DASHBOARD_CACHE_DIR:-/tmp/murano}
 
 MURANO_REPOSITORY_URL=${MURANO_REPOSITORY_URL:-'http://apps.openstack.org/api/v1/murano_repo/liberty/'}
 
-# Functions
-# ---------
-
-function remove_config_block() {
-    local config_file="$1"
-    local label="$2"
-
-    if [[ -f "$config_file" ]] && [[ -n "$label" ]]; then
-        sed -e "/^#${label}_BEGIN/,/^#${label}_END/ d" -i "$config_file"
-    fi
-}
-
-
 # Entry points
 # ------------
 
 # configure_murano_dashboard() - Set config files, create data dirs, etc
 function configure_murano_dashboard() {
-    remove_config_block "$HORIZON_CONFIG" "MURANO_CONFIG_SECTION"
-
     configure_local_settings_py
 
     restart_apache_server
@@ -361,33 +490,34 @@ function configure_local_settings_py() {
 
     mkdir_chown_stack "$MURANO_DASHBOARD_CACHE_DIR"
 
-    # Write changes for dashboard config to a separate file
-    cat << EOF >> "$horizon_config_part"
-
-#MURANO_CONFIG_SECTION_BEGIN
-#-------------------------------------------------------------------------------
-METADATA_CACHE_DIR = '$MURANO_DASHBOARD_CACHE_DIR'
-DATABASES = {
-    'default': {
-        'ENGINE': 'django.db.backends.sqlite3',
-        'NAME': os.path.join('$MURANO_DASHBOARD_DIR', 'openstack-dashboard.sqlite')
-    }
-}
-SESSION_ENGINE = 'django.contrib.sessions.backends.db'
-MURANO_REPO_URL = '$MURANO_REPOSITORY_URL'
-#-------------------------------------------------------------------------------
-#MURANO_CONFIG_SECTION_END
-
-EOF
-
-    cat "$horizon_config_part" >> "$HORIZON_LOCAL_CONFIG"
+    if is_murano_backend_glare; then
+    # Make Murano use GlARe only if MURANO_USE_GLARE set to True and GlARe
+    # service is enabled
+        local murano_use_glare=True
+    else
+        local murano_use_glare=False
+    fi
 
     if [[ -f "$HORIZON_LOCAL_CONFIG" ]]; then
         sed -e "s/\(^\s*OPENSTACK_HOST\s*=\).*$/\1 '$HOST_IP'/" -i "$HORIZON_LOCAL_CONFIG"
     fi
 
     # Install Murano as plugin for Horizon
-    ln -sf $MURANO_DASHBOARD_DIR/muranodashboard/local/_50_murano.py $HORIZON_DIR/openstack_dashboard/local/enabled/
+    ln -sf $MURANO_DASHBOARD_DIR/muranodashboard/local/enabled/*.py $HORIZON_DIR/openstack_dashboard/local/enabled/
+
+    # Install setting to Horizon
+    ln -sf $MURANO_DASHBOARD_DIR/muranodashboard/local/local_settings.d/*.py $HORIZON_DIR/openstack_dashboard/local/local_settings.d/
+
+    # Install murano RBAC policy to Horizon
+    ln -sf $MURANO_DASHBOARD_DIR/muranodashboard/conf/murano_policy.json $HORIZON_DIR/openstack_dashboard/conf/
+
+    # Change Murano dashboard settings
+    sed -e "s/\(^\s*MURANO_USE_GLARE\s*=\).*$/\1 $murano_use_glare/" -i $HORIZON_DIR/openstack_dashboard/local/local_settings.d/_50_murano.py
+    sed -e "s%\(^\s*MURANO_REPO_URL\s*=\).*$%\1 '$MURANO_REPOSITORY_URL'%" -i $HORIZON_DIR/openstack_dashboard/local/local_settings.d/_50_murano.py
+    sed -e "s%\(^\s*'NAME':\).*$%\1 os.path.join('$MURANO_DASHBOARD_DIR', 'openstack-dashboard.sqlite')%" -i $HORIZON_DIR/openstack_dashboard/local/local_settings.d/_50_murano.py
+    echo -e $"\nMETADATA_CACHE_DIR = '$MURANO_DASHBOARD_CACHE_DIR'" | sudo tee -a $HORIZON_DIR/openstack_dashboard/local/local_settings.d/_50_murano.py
+    echo -e $"\nGLARE_API_URL = '$GLANCE_SERVICE_PROTOCOL://$GLANCE_GLARE_HOSTPORT'" | sudo tee -a $HORIZON_DIR/openstack_dashboard/local/local_settings.d/_50_murano.py
+
 }
 
 # init_murano_dashboard() - Initialize databases, etc.
@@ -401,6 +531,10 @@ function init_murano_dashboard() {
     python "$horizon_manage_py" compress --force
     python "$horizon_manage_py" migrate --noinput
 
+    # Compile message for murano-dashboard
+    cd $MURANO_DASHBOARD_DIR/muranodashboard
+    python "$horizon_manage_py" compilemessages
+
     restart_apache_server
 }
 
@@ -419,8 +553,12 @@ function install_murano_dashboard() {
 # runs that a clean run would need to clean up
 function cleanup_murano_dashboard() {
     echo_summary "Cleanup Murano Dashboard"
-    remove_config_block "$HORIZON_CONFIG" "MURANO_CONFIG_SECTION"
+
     rm $HORIZON_DIR/openstack_dashboard/local/enabled/_50_murano.py
+
+    rm $HORIZON_DIR/openstack_dashboard/local/local_settings.d/_50_murano.py
+
+    rm $HORIZON_DIR/openstack_dashboard/conf/murano_policy.json
 }
 
 # Main dispatcher
@@ -440,7 +578,7 @@ if is_service_enabled murano; then
             configure_murano_dashboard
         fi
         if is_service_enabled murano-cfapi; then
-            configure_service_broker
+            configure_murano_cfapi
         fi
     elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
         echo_summary "Initializing Murano"
@@ -449,10 +587,14 @@ if is_service_enabled murano; then
             init_murano_dashboard
         fi
         start_murano
+        if is_murano_backend_glare; then
+            restart_glare_service
+        fi
         if is_service_enabled murano-cfapi; then
+            init_murano_cfapi
             start_service_broker
         fi
-
+        setup_core_library
         configure_murano_tempest_plugin
 
         # Give Murano some time to Start
