@@ -18,14 +18,12 @@ and/or Accept headers and attempts to negotiate an API controller to
 return
 """
 
-from oslo_config import cfg
 from oslo_log import log as logging
 
 from murano.api import versions
 from murano.common.i18n import _LW
 from murano.common import wsgi
 
-CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
@@ -63,7 +61,9 @@ class VersionNegotiationFilter(wsgi.Middleware):
         return None
 
     def _match_version_string(self, subject):
-        """Given a string, tries to match a major and/or
+        """Tries to match major and/or minor version
+
+           Given a string, tries to match a major and/or
            minor version number.
 
            :param subject: The string to check
@@ -77,7 +77,9 @@ class VersionNegotiationFilter(wsgi.Middleware):
         return major_version
 
     def _pop_path_info(self, req):
-        """'Pops' off the next segment of PATH_INFO, returns the popped
+        """Returns the popped off next segment
+
+           'Pops' off the next segment of PATH_INFO, returns the popped
            segment. Do NOT push it onto SCRIPT_NAME.
         """
         path = req.path_info
