@@ -28,9 +28,10 @@ import six
 from webob import exc
 
 import murano.api.v1
-from murano.api.v1 import schemas
+from murano.api.v1 import validation_schemas
 from murano.common import exceptions
 from murano.common import policy
+import murano.common.utils as murano_utils
 from murano.common import wsgi
 from murano.db.catalog import api as db_api
 from murano.common.i18n import _, _LW
@@ -46,6 +47,7 @@ SUPPORTED_PARAMS = murano.api.v1.SUPPORTED_PARAMS
 LIST_PARAMS = murano.api.v1.LIST_PARAMS
 ORDER_VALUES = murano.api.v1.ORDER_VALUES
 PKG_PARAMS_MAP = murano.api.v1.PKG_PARAMS_MAP
+OPERATOR_VALUES = murano.api.v1.OPERATOR_VALUES
 
 
 def _check_content_type(req, content_type):
@@ -67,7 +69,17 @@ def _get_filters(query_params):
             continue
 
         if k in LIST_PARAMS:
-            filters.setdefault(k, []).append(v)
+            if v.startswith('in:') and k in OPERATOR_VALUES:
+                in_value = v[len('in:'):]
+                try:
+                    filters[k] = murano_utils.split_for_quotes(in_value)
+                except ValueError as err:
+                    LOG.warning(_LW("Search by parameter '{name}' "
+                                    "caused an {message} error."
+                                    "Skipping it.").format(name=k,
+                                                           message=err))
+            else:
+                filters.setdefault(k, []).append(v)
         else:
             filters[k] = v
         order_by = filters.get('order_by', [])
@@ -81,11 +93,13 @@ def _get_filters(query_params):
 
 
 def _validate_body(body):
-    """Check multipart/form-data has two parts: text (which is json string and
-       should parsed into dictionary in serializer) and file, which stores as
-       cgi.FieldStorage instance. Also validate file size doesn't exceed
-       the limit: seek to the end of the file, get the position of EOF and
-       reset the file position to the beginning
+    """Check multipart/form-data has two parts
+
+    Check multipart/form-data has two parts: text (which is json string and
+    should parsed into dictionary in serializer) and file, which stores as
+    cgi.FieldStorage instance. Also validate file size doesn't exceed
+    the limit: seek to the end of the file, get the position of EOF and
+    reset the file position to the beginning
     """
     def check_file_size(f):
         mb_limit = CONF.murano.package_size_limit
@@ -98,7 +112,7 @@ def _validate_body(body):
                 'Uploading file is too large. '
                 'The limit is {0} Mb').format(mb_limit))
 
-    if len(body.keys()) > 2:
+    if len(body) > 2:
         msg = _("'multipart/form-data' request body should contain 1 or 2 "
                 "parts: json string and zip archive. Current body consists "
                 "of {amount} part(s)").format(amount=len(body.keys()))
@@ -142,7 +156,9 @@ class Controller(object):
             return value
 
     def update(self, req, body, package_id):
-        """List of allowed changes:
+        """List of allowed changes
+
+        List of allowed changes:
             { "op": "add", "path": "/tags", "value": [ "foo", "bar" ] }
             { "op": "add", "path": "/categories", "value": [ "foo", "bar" ] }
             { "op": "remove", "path": "/tags" }
@@ -208,8 +224,10 @@ class Controller(object):
         return result
 
     def upload(self, req, body=None):
-        """Upload new file archive for the new package
-           together with package metadata.
+        """Upload new file archive
+
+        Upload new file archive for the new package
+        together with package metadata.
         """
         policy.check("upload_package", req.context)
 
@@ -217,7 +235,8 @@ class Controller(object):
         file_obj, package_meta = _validate_body(body)
         if package_meta:
             try:
-                jsonschema.validate(package_meta, schemas.PKG_UPLOAD_SCHEMA)
+                jsonschema.validate(package_meta,
+                                    validation_schemas.PKG_UPLOAD_SCHEMA)
             except jsonschema.ValidationError as e:
                 msg = _("Package schema is not valid: {reason}").format(
                     reason=e)
@@ -281,7 +300,7 @@ class Controller(object):
             with tempfile.NamedTemporaryFile() as tempf:
                 for chunk in blob_data:
                     tempf.write(chunk)
-                tempf.flush()
+                tempf.file.flush()
                 os.fsync(tempf.file.fileno())
                 with load_utils.load_from_file(tempf.name, target_dir=None,
                                                drop_dir=True) as pkg:
@@ -327,7 +346,9 @@ class Controller(object):
         return {'categories': [category.name for category in categories]}
 
     def list_categories(self, req):
-        """List all categories with pagination and sorting
+        """List all categories
+
+        List all categories with pagination and sorting
            Acceptable filter params:
            :param sort_keys: an array of fields used to sort the list
            :param sort_dir: the direction of the sort ('asc' or 'desc')
@@ -341,7 +362,7 @@ class Controller(object):
                 if key not in valid_query_params:
                     raise exc.HTTPBadRequest(
                         _('Bad value passed to filter. '
-                          'Got {key}, exected:{valid}').format(
+                          'Got {key}, expected:{valid}').format(
                             key=key, valid=', '.join(valid_query_params)))
                 if key == 'sort_keys':
                     available_sort_keys = ['name', 'created',
