@@ -20,8 +20,10 @@ from murano.api.v1 import deployments
 from murano.api.v1 import environments
 from murano.api.v1 import instance_statistics
 from murano.api.v1 import request_statistics
+from murano.api.v1 import schemas
 from murano.api.v1 import services
 from murano.api.v1 import sessions
+from murano.api.v1 import static_actions
 from murano.api.v1 import template_applications
 from murano.api.v1 import templates
 from murano.common import wsgi
@@ -139,6 +141,10 @@ class API(wsgi.Router):
                        controller=applications_resource,
                        action='post',
                        conditions={'method': ['POST']}, path='')
+        mapper.connect('/templates/{env_template_id}/services/{path:.*?}',
+                       controller=applications_resource,
+                       action='put',
+                       conditions={'method': ['PUT']}, path='')
         mapper.connect('/templates/{env_template_id}/services/{path:.*?}',
                        controller=applications_resource,
                        action='delete',
@@ -202,6 +208,12 @@ class API(wsgi.Router):
                        action='get_result',
                        conditions={'method': ['GET']})
 
+        static_actions_resource = static_actions.create_resource()
+        mapper.connect('/actions',
+                       controller=static_actions_resource,
+                       action='execute',
+                       conditions={'method': ['POST']})
+
         catalog_resource = catalog.create_resource()
         mapper.connect('/catalog/packages/categories',
                        controller=catalog_resource,
@@ -264,5 +276,14 @@ class API(wsgi.Router):
                        controller=req_stats_resource,
                        action='get',
                        conditions={'method': ['GET']})
+        schemas_resource = schemas.create_resource()
+        mapper.connect('/schemas/{class_name}/{method_names}',
+                       controller=schemas_resource,
+                       action='get_schema',
+                       conditions={'method': ['GET']})
+        mapper.connect('/schemas/{class_name}',
+                       controller=schemas_resource,
+                       action='get_schema',
+                       conditions={'method': ['GET']})
 
         super(API, self).__init__(mapper)
