@@ -35,9 +35,8 @@ from oslo_service import service
 
 from murano.api.v1 import request_statistics
 from murano.common import app_loader
-from murano.common import config
+from murano.common import cf_config as config
 from murano.common import policy
-from murano.common import server
 from murano.common import wsgi
 
 CONF = cfg.CONF
@@ -65,9 +64,6 @@ def main():
 
         launcher.launch_service(wsgi.Service(cfapp, cfport, cfhost))
 
-        launcher.launch_service(server.ApiService())
-        launcher.launch_service(server.NotificationService())
-
         launcher.wait()
     except RuntimeError as e:
         sys.stderr.write("ERROR: %s\n" % e)
