@@ -1,4 +1,4 @@
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -166,6 +166,9 @@ murano_opts = [
                     'pagination request',
                deprecated_group='packages_opts'),
 
+    cfg.IntOpt('api_workers',
+               help=_('Number of API workers')),
+
 ]
 
 networking_opts = [
@@ -198,7 +201,11 @@ networking_opts = [
 
     cfg.StrOpt('network_config_file', default='netconfig.yaml',
                help='If provided networking configuration will be taken '
-                    'from this file')
+                    'from this file'),
+
+    cfg.StrOpt('driver', default=None, choices=['neutron', 'nova'],
+               help='Network driver to use. Options are neutron or nova.'
+                    'If not provided, the driver will be detected.'),
 ]
 
 stats_opts = [
@@ -220,8 +227,10 @@ engine_opts = [
     cfg.IntOpt('agent_timeout', default=3600,
                help=_('Time for waiting for a response from murano agent '
                       'during the deployment')),
-    cfg.IntOpt('workers',
-               help=_('Number of workers')),
+    cfg.IntOpt('engine_workers',
+               deprecated_opts=[cfg.DeprecatedOpt('workers',
+                                                  group='engine')],
+               help=_('Number of engine workers')),
 
     cfg.ListOpt('load_packages_from', default=[],
                 help=_('List of directories to load local packages from. '
