@@ -53,7 +53,6 @@ class PluginLoader(object):
             LOG.warning(_LW("Entry-point 'name' {name} is invalid").format(
                 name=name))
             return
-        name = "%s.%s" % (self.namespace, name)
         name_map.setdefault(name, []).append(dist_name)
         if dist_name in self.packages:
             package = self.packages[dist_name]
@@ -135,8 +134,8 @@ class MuranoPackage(murano_package.MuranoPackage):
                                 "ignored") %
                             dict(class_name=class_name,
                             name=getattr(clazz, "_murano_class_name")))
-            LOG.debug("Registering '%s' from '%s' in class loader"
-                      % (class_name, package_definition.name))
+            LOG.debug("Registering '%s' from '%s' in class loader",
+                      class_name, package_definition.name)
             self.register_class(clazz, class_name)
 
     def get_resource(self, name):
