@@ -34,8 +34,3 @@ class RequestContext(context.RequestContext):
         self.is_admin = is_admin
         if self.is_admin is None:
             self.is_admin = policy.check_is_admin(self)
-
-    def to_dict(self):
-        d = super(RequestContext, self).to_dict()
-        d.setdefault('roles', self.roles)
-        return d
