@@ -12,6 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
+
 import six
 
 from murano.dsl import dsl_types
@@ -19,7 +21,7 @@ from murano.dsl import dsl_types
 
 class AttributeStore(object):
     def __init__(self):
-        self._attributes = {}
+        self._attributes = collections.defaultdict(lambda: {})
 
     @staticmethod
     def _get_attribute_key(tagged_object, owner_type, name):
@@ -27,32 +29,39 @@ class AttributeStore(object):
             owner_type = owner_type.type
         if isinstance(tagged_object, dsl_types.MuranoObjectInterface):
             tagged_object = tagged_object.object
-        return tagged_object.object_id, owner_type.name, name
+        return tagged_object.object_id, (owner_type.name, name)
 
     def get(self, tagged_object, owner_type, name):
-        return self._attributes.get(self._get_attribute_key(
-            tagged_object, owner_type, name))
+        key1, key2 = self._get_attribute_key(tagged_object, owner_type, name)
+        return self._attributes[key1].get(key2)
 
     def set(self, tagged_object, owner_type, name, value):
         if isinstance(value, dsl_types.MuranoObjectInterface):
             value = value.id
         elif isinstance(value, dsl_types.MuranoObject):
             value = value.object_id
-        key = self._get_attribute_key(tagged_object, owner_type, name)
+        key1, key2 = self._get_attribute_key(tagged_object, owner_type, name)
         if value is None:
-            self._attributes.pop(key, None)
+            self._attributes[key1].pop(key2, None)
         else:
-            self._attributes[key] = value
+            self._attributes[key1][key2] = value
 
     def serialize(self, known_objects):
         return [
-            [key[0], key[1], key[2], value]
-            for key, value
-            in six.iteritems(self._attributes)
-            if key[0] in known_objects
+            [key1, key2[0], key2[1], value]
+            for key1, inner in six.iteritems(self._attributes)
+            for key2, value in six.iteritems(inner)
+            if key1 in known_objects
         ]
 
     def load(self, data):
         for item in data:
             if item[3] is not None:
-                self._attributes[(item[0], item[1], item[2])] = item[3]
+                self._attributes[item[0]][(item[1], item[2])] = item[3]
+
+    def forget_object(self, obj):
+        if isinstance(obj, dsl_types.MuranoObjectInterface):
+            obj = obj.id
+        elif isinstance(obj, dsl_types.MuranoObject):
+            obj = obj.object_id
+        self._attributes.pop(obj, None)
