@@ -16,6 +16,7 @@ import collections
 import sys
 import weakref
 
+from oslo_log import log as logging
 import six
 from yaql.language import specs
 
@@ -31,6 +32,7 @@ from murano.dsl import virtual_exceptions
 from murano.dsl import yaql_integration
 
 
+LOG = logging.getLogger(__name__)
 macros.register()
 virtual_exceptions.register()
 
@@ -51,6 +53,7 @@ class MuranoMethod(dsl_types.MuranoMethod, meta.MetaProvider):
                 self._body = yaql_integration.get_function_definition(
                     payload, self_ref, original_name)
             self._arguments_scheme = None
+            self._scope = self._body.meta.get(constants.META_SCOPE)
             if declaring_type.extension_class and any((
                     helpers.inspect_is_static(
                         declaring_type.extension_class, original_name),
@@ -67,6 +70,9 @@ class MuranoMethod(dsl_types.MuranoMethod, meta.MetaProvider):
                 if self._usage not in dsl_types.MethodUsages.InstanceMethods:
                     raise ValueError(
                         'Invalid Usage for instance method ' + self.name)
+                self._resolve_usage_and_scope()
+            if self._scope is None:
+                self._scope = dsl_types.MethodScopes.Session
             if (self._body.name.startswith('#') or
                     self._body.name.startswith('*')):
                 raise ValueError(
@@ -80,18 +86,45 @@ class MuranoMethod(dsl_types.MuranoMethod, meta.MetaProvider):
             self._body = macros.MethodBlock(payload.get('Body'), name)
             self._usage = payload.get(
                 'Usage') or dsl_types.MethodUsages.Runtime
+            self._scope = payload.get('Scope')
+            self._resolve_usage_and_scope()
+            if self._scope is None:
+                self._scope = dsl_types.MethodScopes.Session
             arguments_scheme = helpers.list_value(payload.get('Arguments'))
             if isinstance(arguments_scheme, dict):
                 arguments_scheme = [{key: value} for key, value in
                                     six.iteritems(arguments_scheme)]
             self._arguments_scheme = collections.OrderedDict()
+            seen_varargs = False
+            seen_kwargs = False
+            args_order_error = False
             for record in arguments_scheme:
-                if (not isinstance(record, dict) or
-                        len(record) > 1):
-                    raise ValueError()
+                if not isinstance(record, dict) or len(record) > 1:
+                    raise exceptions.DslSyntaxError(
+                        'Invalid arguments declaration')
                 name = list(record.keys())[0]
-                self._arguments_scheme[name] = MuranoMethodArgument(
+                argument = MuranoMethodArgument(
                     self, self.name, name, record[name])
+                usage = argument.usage
+                if (usage == dsl_types.MethodArgumentUsages.Standard and
+                        (seen_kwargs or seen_varargs)):
+                    args_order_error = True
+                elif usage == dsl_types.MethodArgumentUsages.VarArgs:
+                    if seen_kwargs or seen_varargs:
+                        args_order_error = True
+                    seen_varargs = True
+                elif usage == dsl_types.MethodArgumentUsages.KwArgs:
+                    if seen_kwargs:
+                        args_order_error = True
+                    seen_kwargs = True
+
+                if args_order_error:
+                    raise exceptions.DslSyntaxError(
+                        'Invalid argument order in method {0}'.format(
+                            self.name))
+                else:
+                    self._arguments_scheme[name] = argument
+
             self._meta = meta.MetaData(
                 payload.get('Meta'),
                 dsl_types.MetaTargets.Method,
@@ -100,6 +133,18 @@ class MuranoMethod(dsl_types.MuranoMethod, meta.MetaProvider):
         self._instance_stub, self._static_stub = \
             yaql_integration.build_stub_function_definitions(self_ref)
 
+    def _resolve_usage_and_scope(self):
+        if self._usage == dsl_types.MethodUsages.Action:
+            runtime_version = self.declaring_type.package.runtime_version
+            if runtime_version > constants.RUNTIME_VERSION_1_3:
+                LOG.warning('"Usage: Action" is deprecated, '
+                            'use "Scope: Public" instead')
+            if self._scope == dsl_types.MethodScopes.Session:
+                raise ValueError(
+                    'Both "Usage: Action" and "Scope: Session" are '
+                    'provided for method ' + self.name)
+            self._scope = dsl_types.MethodScopes.Public
+
     @property
     def name(self):
         return self._name
@@ -128,6 +173,14 @@ class MuranoMethod(dsl_types.MuranoMethod, meta.MetaProvider):
     def usage(self, value):
         self._usage = value
 
+    @property
+    def scope(self):
+        return self._scope
+
+    @scope.setter
+    def scope(self, value):
+        self._scope = value
+
     @property
     def body(self):
         return self._body
@@ -136,6 +189,11 @@ class MuranoMethod(dsl_types.MuranoMethod, meta.MetaProvider):
     def is_static(self):
         return self.usage in dsl_types.MethodUsages.StaticMethods
 
+    @property
+    def is_action(self):
+        return (self.scope == dsl_types.MethodScopes.Public or
+                self.usage == dsl_types.MethodUsages.Action)
+
     def get_meta(self, context):
         def meta_producer(cls):
             method = cls.methods.get(self.name)
@@ -144,7 +202,7 @@ class MuranoMethod(dsl_types.MuranoMethod, meta.MetaProvider):
             return method._meta
 
         if self._meta_values is None:
-            executor = helpers.get_executor(context)
+            executor = helpers.get_executor()
             context = executor.create_type_context(
                 self.declaring_type, caller_context=context)
             self._meta_values = meta.merge_providers(
@@ -155,8 +213,7 @@ class MuranoMethod(dsl_types.MuranoMethod, meta.MetaProvider):
         return 'MuranoMethod({0}::{1})'.format(
             self.declaring_type.name, self.name)
 
-    def invoke(self, executor, this, args, kwargs, context=None,
-               skip_stub=False):
+    def invoke(self, this, args, kwargs, context=None, skip_stub=False):
         if isinstance(this, dsl.MuranoObjectInterface):
             this = this.object
         if this and not self.declaring_type.is_compatible(this):
@@ -168,6 +225,7 @@ class MuranoMethod(dsl_types.MuranoMethod, meta.MetaProvider):
             this = this.cast(self.declaring_type)
         else:
             this = self.declaring_type
+        executor = helpers.get_executor()
         return executor.invoke_method(
             self, this, context, args, kwargs, skip_stub)
 
@@ -183,6 +241,14 @@ class MuranoMethodArgument(dsl_types.MuranoMethodArgument, typespec.Spec,
         self._meta = meta.MetaData(
             declaration.get('Meta'),
             dsl_types.MetaTargets.Argument, self.murano_method.declaring_type)
+        self._usage = declaration.get('Usage') or \
+            dsl_types.MethodArgumentUsages.Standard
+
+        if self._usage not in dsl_types.MethodArgumentUsages.All:
+            raise exceptions.DslSyntaxError(
+                'Unknown usage {0}. Must be one of ({1})'.format(
+                    self._usage, ', '.join(dsl_types.MethodArgumentUsages.All)
+                ))
 
     def transform(self, value, this, *args, **kwargs):
         try:
@@ -207,8 +273,12 @@ class MuranoMethodArgument(dsl_types.MuranoMethodArgument, typespec.Spec,
     def name(self):
         return self._arg_name
 
+    @property
+    def usage(self):
+        return self._usage
+
     def get_meta(self, context):
-        executor = helpers.get_executor(context)
+        executor = helpers.get_executor()
         context = executor.create_type_context(
             self.murano_method.declaring_type, caller_context=context)
 
