@@ -91,7 +91,7 @@ class ContractedValue(yaqltypes.GenericType):
                 context)
 
         def checker(value, context, *args, **kwargs):
-            return value_spec.validate(value, context)
+            return value_spec.check_type(value, context)
 
         super(ContractedValue, self).__init__(
             True, checker if with_check else None, converter)
@@ -240,11 +240,10 @@ def _build_native_stub_function_definitions(murano_method):
     @specs.parameter('__receiver', yaqltypes.NotOfType(
         dsl_types.MuranoTypeReference))
     def payload(__context, __receiver, *args, **kwargs):
-        executor = helpers.get_executor(__context)
         args = tuple(dsl.to_mutable(arg, engine) for arg in args)
         kwargs = dsl.to_mutable(kwargs, engine)
         return helpers.evaluate(murano_method.invoke(
-            executor, __receiver, args, kwargs, __context, True), __context)
+            __receiver, args, kwargs, __context, True), __context)
 
     @specs.method
     @specs.name(murano_method.name)
@@ -252,23 +251,21 @@ def _build_native_stub_function_definitions(murano_method):
     @specs.parameter('__receiver', yaqltypes.NotOfType(
         dsl_types.MuranoTypeReference))
     def extension_payload(__context, __receiver, *args, **kwargs):
-        executor = helpers.get_executor(__context)
         args = tuple(dsl.to_mutable(arg, engine) for arg in args)
         kwargs = dsl.to_mutable(kwargs, engine)
         return helpers.evaluate(murano_method.invoke(
-            executor, murano_method.declaring_type,
-            (__receiver,) + args, kwargs, __context, True), __context)
+            murano_method.declaring_type, (__receiver,) + args, kwargs,
+            __context, True), __context)
 
     @specs.method
     @specs.name(murano_method.name)
     @specs.meta(constants.META_MURANO_METHOD, murano_method)
     @specs.parameter('__receiver', dsl_types.MuranoTypeReference)
     def static_payload(__context, __receiver, *args, **kwargs):
-        executor = helpers.get_executor(__context)
         args = tuple(dsl.to_mutable(arg, engine) for arg in args)
         kwargs = dsl.to_mutable(kwargs, engine)
         return helpers.evaluate(murano_method.invoke(
-            executor, __receiver, args, kwargs, __context, True), __context)
+            __receiver, args, kwargs, __context, True), __context)
 
     if murano_method.usage in dsl_types.MethodUsages.InstanceMethods:
         return specs.get_function_definition(payload), None
@@ -297,9 +294,7 @@ def _build_mpl_stub_function_definitions(murano_method):
 
 def _create_instance_mpl_stub(murano_method):
     def payload(__context, __receiver, *args, **kwargs):
-        executor = helpers.get_executor(__context)
-        return murano_method.invoke(
-            executor, __receiver, args, kwargs, __context, True)
+        return murano_method.invoke(__receiver, args, kwargs, __context, True)
     fd = _create_basic_mpl_stub(murano_method, 1, payload, False)
 
     receiver_type = dsl.MuranoObjectParameter(
@@ -310,9 +305,7 @@ def _create_instance_mpl_stub(murano_method):
 
 def _create_static_mpl_stub(murano_method):
     def payload(__context, __receiver, *args, **kwargs):
-        executor = helpers.get_executor(__context)
-        return murano_method.invoke(
-            executor, __receiver, args, kwargs, __context, True)
+        return murano_method.invoke(__receiver, args, kwargs, __context, True)
     fd = _create_basic_mpl_stub(murano_method, 1, payload, False)
 
     receiver_type = dsl.MuranoTypeParameter(
@@ -323,10 +316,9 @@ def _create_static_mpl_stub(murano_method):
 
 def _create_extension_mpl_stub(murano_method):
     def payload(__context, __receiver, *args, **kwargs):
-        executor = helpers.get_executor(__context)
         return murano_method.invoke(
-            executor, murano_method.declaring_type,
-            (__receiver,) + args, kwargs, __context, True)
+            murano_method.declaring_type, (__receiver,) + args, kwargs,
+            __context, True)
     return _create_basic_mpl_stub(murano_method, 0, payload, True)
 
 
@@ -335,13 +327,33 @@ def _create_basic_mpl_stub(murano_method, reserve_params, payload,
     fd = specs.FunctionDefinition(
         murano_method.name, payload, is_function=False, is_method=True)
 
-    for i, (name, arg_spec) in enumerate(
-            six.iteritems(murano_method.arguments_scheme), reserve_params + 1):
+    i = reserve_params + 1
+    varargs = False
+    kwargs = False
+    for name, arg_spec in six.iteritems(murano_method.arguments_scheme):
+        position = i
+        if arg_spec.usage == dsl_types.MethodArgumentUsages.VarArgs:
+            name = '*'
+            varargs = True
+        elif arg_spec.usage == dsl_types.MethodArgumentUsages.KwArgs:
+            name = '**'
+            position = None
+            kwargs = True
         p = specs.ParameterDefinition(
             name, ContractedValue(arg_spec, with_check=check_first_arg),
-            position=i, default=dsl.NO_VALUE)
+            position=position, default=dsl.NO_VALUE)
         check_first_arg = False
         fd.parameters[name] = p
+        i += 1
+
+    if not varargs:
+        fd.parameters['*'] = specs.ParameterDefinition(
+            '*',
+            value_type=yaqltypes.PythonType(object, nullable=True),
+            position=i)
+    if not kwargs:
+        fd.parameters['**'] = specs.ParameterDefinition(
+            '**', value_type=yaqltypes.PythonType(object, nullable=True))
 
     fd.set_parameter(specs.ParameterDefinition(
         '__context', yaqltypes.Context(), 0))
@@ -358,17 +370,18 @@ def get_class_factory_definition(cls, murano_class):
         args = tuple(dsl.to_mutable(arg, engine) for arg in args)
         kwargs = dsl.to_mutable(kwargs, engine)
         with helpers.contextual(__context):
-            __context[constants.CTX_NAMES_SCOPE] = \
-                murano_class
-            return helpers.evaluate(cls(*args, **kwargs), __context)
+            __context[constants.CTX_NAMES_SCOPE] = murano_class
+            result = helpers.evaluate(cls(*args, **kwargs), __context)
+            __receiver.object.extension = result
 
-    if '__init__' in cls.__dict__:
+    try:
         fd = specs.get_function_definition(
             helpers.function(cls.__init__),
             parameter_type_func=lambda name: _infer_parameter_type(
                 name, cls.__name__),
             convention=CONVENTION)
-    else:
+    except AttributeError:
+        # __init__ is a slot wrapper inherited from object or other C type
         fd = specs.get_function_definition(lambda self: None)
         fd.meta[constants.META_NO_TRACE] = True
     fd.insert_parameter(specs.ParameterDefinition(
