@@ -21,6 +21,7 @@ import uuid
 import eventlet.event
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_serialization import base64
 import six
 from yaql import specs
 
@@ -46,10 +47,10 @@ class Agent(object):
                       'by the server configuration')
             return
 
-        self._environment = host.find_owner('io.murano.Environment')
+        self._host = host
         self._enabled = True
-        self._queue = str('e%s-h%s' % (
-            self._environment.id, host.id)).lower()
+        env = host.find_owner('io.murano.Environment')
+        self._queue = str('e%s-h%s' % (env.id, host.id)).lower()
 
     @property
     def enabled(self):
@@ -62,7 +63,8 @@ class Agent(object):
                       'by the server configuration')
             return
 
-        with common.create_rmq_client() as client:
+        region = dsl.MuranoObjectInterface.create(self._host().getRegion())
+        with common.create_rmq_client(region) as client:
             client.declare(self._queue, enable_ha=True, ttl=86400000)
 
     def queue_name(self):
@@ -82,14 +84,15 @@ class Agent(object):
 
     def _send(self, template, wait_results, timeout):
         """Send a message over the MQ interface."""
+        region = self._host().getRegion()
         msg_id = template.get('ID', uuid.uuid4().hex)
         if wait_results:
             event = eventlet.event.Event()
-            listener = self._environment['agentListener']
+            listener = region['agentListener']
             listener().subscribe(msg_id, event)
 
         msg = self._prepare_message(template, msg_id)
-        with common.create_rmq_client() as client:
+        with common.create_rmq_client(region) as client:
             client.send(message=msg, key=self._queue)
 
         if wait_results:
@@ -216,8 +219,8 @@ class Agent(object):
         scripts = []
         for script in script_files:
             script_path = os.path.join(scripts_folder, script)
-            scripts.append(resources.string(
-                script_path).encode('base64'))
+            script_path = base64.encode_as_text(script_path) + "\n"
+            scripts.append(resources.string(script_path, binary=True))
         template['Scripts'] = scripts
         return template
 
@@ -287,11 +290,13 @@ class Agent(object):
 
     def _get_body(self, file, resources, folder):
         use_base64 = self._is_base64(file)
-        if use_base64 and file.startswith('<') and file.endswith('>'):
-            file = file[1: -1]
-        body = resources.string(os.path.join(folder, file))
         if use_base64:
-            body = body.encode('base64')
+            path = os.path.join(folder, file[1: -1])
+            body = resources.string(path, binary=True)
+            body = base64.encode_as_text(body) + "\n"
+        else:
+            path = os.path.join(folder, file)
+            body = resources.string(path)
         return body
 
     def _is_base64(self, file):
