@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import copy
+import json
 
 import eventlet
 import heatclient.client as hclient
@@ -24,7 +25,6 @@ import six
 
 from murano.common import auth_utils
 from murano.common.i18n import _LW
-from murano.common import utils
 from murano.dsl import dsl
 from murano.dsl import helpers
 from murano.dsl import session_local_storage
@@ -41,7 +41,7 @@ class HeatStackError(Exception):
 
 @dsl.name('io.murano.system.HeatStack')
 class HeatStack(object):
-    def __init__(self, this, name, description=None):
+    def __init__(self, name, description=None, region_name=None):
         self._name = name
         self._template = None
         self._parameters = {}
@@ -51,7 +51,7 @@ class HeatStack(object):
         self._description = description
         self._last_stack_timestamps = (None, None)
         self._tags = ''
-        self._owner = this.find_owner('io.murano.Environment')
+        self._region_name = region_name
 
     @staticmethod
     def _create_client(session, region_name):
@@ -62,8 +62,7 @@ class HeatStack(object):
 
     @property
     def _client(self):
-        region = None if self._owner is None else self._owner['region']
-        return self._get_client(region)
+        return self._get_client(self._region_name)
 
     @staticmethod
     @session_local_storage.execution_session_memoize
@@ -73,8 +72,7 @@ class HeatStack(object):
 
     def _get_token_client(self):
         ks_session = auth_utils.get_token_client_session(conf=CONF.heat)
-        region = None if self._owner is None else self._owner['region']
-        return self._create_client(ks_session, region)
+        return self._create_client(ks_session, self._region_name)
 
     def current(self):
         if self._template is not None:
@@ -129,9 +127,9 @@ class HeatStack(object):
             err_msg = ("Currently only heat_template_version %s "
                        "is supported." % HEAT_TEMPLATE_VERSION)
             raise HeatStackError(err_msg)
-        self.current()
+        current = self.current()
         self._template = helpers.merge_dicts(self._template, template)
-        self._applied = False
+        self._applied = self._template == current and self._applied
 
     @staticmethod
     def _remove_system_params(parameters):
@@ -213,7 +211,7 @@ class HeatStack(object):
             self._template['description'] = self._description
 
         template = copy.deepcopy(self._template)
-        LOG.debug('Pushing: {template}'.format(template=template))
+        LOG.debug('Pushing: {template}'.format(template=json.dumps(template)))
 
         while True:
             try:
@@ -254,7 +252,7 @@ class HeatStack(object):
             else:
                 break
 
-        self._applied = not utils.is_different(self._template, template)
+        self._applied = self._template == template
 
     def delete(self):
         while True:
