@@ -35,12 +35,13 @@ LOG = logging.getLogger(__name__)
 
 @dsl.name('io.murano.system.NetworkExplorer')
 class NetworkExplorer(object):
-    def __init__(self, this):
+    def __init__(self, this, region_name=None):
         session = helpers.get_execution_session()
         self._project_id = session.project_id
         self._settings = CONF.networking
         self._available_cidrs = self._generate_possible_cidrs()
-        self._owner = this.find_owner('io.murano.Environment')
+        self._region = this.find_owner('io.murano.CloudRegion')
+        self._region_name = region_name
 
     @staticmethod
     @session_local_storage.execution_session_memoize
@@ -52,7 +53,8 @@ class NetworkExplorer(object):
 
     @property
     def _client(self):
-        region = None if self._owner is None else self._owner['region']
+        region = self._region_name or (
+            None if self._region is None else self._region['name'])
         return self._get_client(region)
 
     # NOTE(starodubcevna): to avoid simultaneous router requests we use retry
@@ -107,10 +109,11 @@ class NetworkExplorer(object):
         return router_id
 
     def get_available_cidr(self, router_id, net_id):
-        """Uses hash of network IDs to minimize the collisions:
-        different nets will attempt to pick different cidrs out of available
+        """Uses hash of network IDs to minimize the collisions
+
+        Different nets will attempt to pick different cidrs out of available
         range.
-        If the cidr is taken will pick another one
+        If the cidr is taken will pick another one.
         """
         taken_cidrs = self._get_cidrs_taken_by_router(router_id)
         id_hash = hash(net_id)
@@ -190,3 +193,6 @@ class NetworkExplorer(object):
 
     def list_ports(self):
         return self._client.list_ports()['ports']
+
+    def list_neutron_extensions(self):
+        return self._client.list_extensions()['extensions']
