@@ -20,7 +20,6 @@ try:
     import congressclient.v1.client as congress_client
 except ImportError as congress_client_import_error:
     congress_client = None
-from oslo_config import cfg
 from oslo_log import log as logging
 
 from murano.common import auth_utils
@@ -30,7 +29,6 @@ from murano.policy.modify.actions import action_manager as am
 
 
 LOG = logging.getLogger(__name__)
-CONF = cfg.CONF
 
 
 class ValidationError(Exception):
@@ -153,12 +151,17 @@ class ModelPolicyEnforcer(object):
 
     @staticmethod
     def _parse_simulation_result(query, env_id, results):
-        """Transforms list of strings in format
-            ['predeploy_errors("env_id_1", "obj_id_1", "message1")',
-            'predeploy_errors("env_id_2", "obj_id_2", "message2")']
-        to list of strings with message only filtered to provided
+        """Transforms the list of results
+
+        Transforms a list of strings in format:
+
+        ['predeploy_errors("env_id_1", "obj_id_1", "message1")',
+        'predeploy_errors("env_id_2", "obj_id_2", "message2")']
+
+        to a list of strings with message only filtered to provided
         env_id (e.g. 'env_id_1'):
-            ['message2']
+
+        ['message2']
         """
 
         messages = []
