@@ -83,7 +83,9 @@ class ActionServices(object):
 
     @staticmethod
     def find_action(model, action_id):
-        """Traverses object model looking for an object definition
+        """Find_action for object def with specified action
+
+        Traverses object model looking for an object definition
         containing specified action
 
         :param model: object model
