@@ -15,24 +15,27 @@
 
 import cgi
 import imghdr
-import json
 import os
 import uuid
 
 import mock
+from oslo_serialization import jsonutils
 from oslo_utils import timeutils
+import six
 from six.moves import cStringIO
 from six.moves import range
 
 from murano.api.v1 import catalog
 from murano.db.catalog import api as db_catalog_api
 from murano.db import models
+from murano.packages import exceptions
 from murano.packages import load_utils
 import murano.tests.unit.api.base as test_base
 import murano.tests.unit.utils as test_utils
 
 
 class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
+
     def setUp(self):
         super(TestCatalogApi, self).setUp()
         self.controller = catalog.Controller()
@@ -164,7 +167,9 @@ class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
         self.assertEqual(2, len(result['packages']))
 
     def test_packages_filter_by_id(self):
-        """GET /catalog/packages with parameter "id" returns packages
+        """Test that packages are filtered by ID
+
+        GET /catalog/packages with parameter "id" returns packages
         filtered by id.
         """
         self._set_policy_rules(
@@ -198,8 +203,179 @@ class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
 
         self.assertEqual(expected_package.id, found_package['id'])
 
+    def test_packages_filter_by_in_category(self):
+        """Test that packages are filtered by in:cat1,cat2,cat3
+
+        GET /catalog/packages with parameter "category=in:cat1,cat2,cat3"
+        returns packages filtered by category.
+        """
+        names = ['cat1', 'cat2', 'cat3', 'cat4']
+        for name in names:
+            db_catalog_api.category_add(name)
+        self._set_policy_rules(
+            {'get_package': '',
+             'manage_public_package': ''}
+        )
+        _, package1_data = self._test_package()
+        _, package2_data = self._test_package()
+        _, package3_data = self._test_package()
+
+        package1_data['fully_qualified_name'] += '_1'
+        package1_data['name'] += '_1'
+        package1_data['class_definitions'] = (u'test.mpl.v1.app.Thing1',)
+        package1_data['categories'] = ('cat1', 'cat2')
+
+        package2_data['fully_qualified_name'] += '_2'
+        package2_data['name'] += '_2'
+        package2_data['class_definitions'] = (u'test.mpl.v1.app.Thing2',)
+        package2_data['categories'] = ('cat2', 'cat3')
+
+        package3_data['fully_qualified_name'] += '_3'
+        package3_data['name'] += '_3'
+        package3_data['class_definitions'] = (u'test.mpl.v1.app.Thing3',)
+        package3_data['categories'] = ('cat2', 'cat4')
+
+        expected_packages = [db_catalog_api.package_upload(package1_data, ''),
+                             db_catalog_api.package_upload(package2_data, '')]
+        db_catalog_api.package_upload(package3_data, '')
+
+        categories_in = "in:cat1,cat3"
+
+        req = self._get('/catalog/packages',
+                        params={'category': categories_in})
+        self.expect_policy_check('get_package')
+        self.expect_policy_check('manage_public_package')
+
+        res = req.get_response(self.api)
+        self.assertEqual(200, res.status_code)
+        self.assertEqual(2, len(res.json['packages']))
+
+        found_packages = res.json['packages']
+
+        self.assertEqual([pack.id for pack in expected_packages],
+                         [pack['id'] for pack in found_packages])
+
+    def test_packages_filter_by_in_tag(self):
+        """Test that packages are filtered by in:tag1,tag2,tag3
+
+        GET /catalog/packages with parameter "tag=in:tag1,tag2,tag3"
+        returns packages filtered by category.
+        """
+        self._set_policy_rules(
+            {'get_package': '',
+             'manage_public_package': ''}
+        )
+        _, package1_data = self._test_package()
+        _, package2_data = self._test_package()
+        _, package3_data = self._test_package()
+
+        package1_data['fully_qualified_name'] += '_1'
+        package1_data['name'] += '_1'
+        package1_data['class_definitions'] = (u'test.mpl.v1.app.Thing1',)
+        package1_data['tags'] = ('tag1', 'tag2')
+
+        package2_data['fully_qualified_name'] += '_2'
+        package2_data['name'] += '_2'
+        package2_data['class_definitions'] = (u'test.mpl.v1.app.Thing2',)
+        package2_data['tags'] = ('tag2', 'tag3')
+
+        package3_data['fully_qualified_name'] += '_3'
+        package3_data['name'] += '_3'
+        package3_data['class_definitions'] = (u'test.mpl.v1.app.Thing3',)
+        package3_data['tags'] = ('tag2', 'tag4')
+
+        expected_packages = [db_catalog_api.package_upload(package1_data, ''),
+                             db_catalog_api.package_upload(package2_data, '')]
+        db_catalog_api.package_upload(package3_data, '')
+
+        tag_in = "in:tag1,tag3"
+
+        req = self._get('/catalog/packages',
+                        params={'tag': tag_in})
+        self.expect_policy_check('get_package')
+        self.expect_policy_check('manage_public_package')
+
+        res = req.get_response(self.api)
+        self.assertEqual(200, res.status_code)
+        self.assertEqual(2, len(res.json['packages']))
+
+        found_packages = res.json['packages']
+
+        self.assertEqual([pack.id for pack in expected_packages],
+                         [pack['id'] for pack in found_packages])
+
+    def test_packages_filter_by_in_id(self):
+        """Test that packages are filtered by in:id1,id2,id3
+
+        GET /catalog/packages with parameter "id=in:id1,id2" returns packages
+        filtered by id.
+        """
+        self._set_policy_rules(
+            {'get_package': '',
+             'manage_public_package': ''}
+        )
+        _, package1_data = self._test_package()
+        _, package2_data = self._test_package()
+        _, package3_data = self._test_package()
+
+        package1_data['fully_qualified_name'] += '_1'
+        package1_data['name'] += '_1'
+        package1_data['class_definitions'] = (u'test.mpl.v1.app.Thing1',)
+        package2_data['fully_qualified_name'] += '_2'
+        package2_data['name'] += '_2'
+        package2_data['class_definitions'] = (u'test.mpl.v1.app.Thing2',)
+        package3_data['fully_qualified_name'] += '_3'
+        package3_data['name'] += '_3'
+        package3_data['class_definitions'] = (u'test.mpl.v1.app.Thing3',)
+
+        expected_packages = [db_catalog_api.package_upload(package1_data, ''),
+                             db_catalog_api.package_upload(package2_data, '')]
+        db_catalog_api.package_upload(package3_data, '')
+
+        id_in = "in:" + ",".join(pack.id for pack in expected_packages)
+
+        req = self._get('/catalog/packages',
+                        params={'id': id_in})
+        self.expect_policy_check('get_package')
+        self.expect_policy_check('manage_public_package')
+
+        res = req.get_response(self.api)
+        self.assertEqual(200, res.status_code)
+
+        self.assertEqual(2, len(res.json['packages']))
+
+        found_packages = res.json['packages']
+
+        self.assertEqual([pack.id for pack in expected_packages],
+                         [pack['id'] for pack in found_packages])
+
+    def test_packages_filter_by_in_id_empty(self):
+        """Test that packages are filtered by "id=in:"
+
+        GET /catalog/packages with parameter "id=in:" returns packages
+        filtered by id, in this case no packages should be returned.
+        """
+        self._set_policy_rules(
+            {'get_package': '',
+             'manage_public_package': ''}
+        )
+        _, package1_data = self._test_package()
+
+        db_catalog_api.package_upload(package1_data, '')
+
+        req = self._get('/catalog/packages', params={'id': "in:"})
+        self.expect_policy_check('get_package')
+        self.expect_policy_check('manage_public_package')
+
+        res = req.get_response(self.api)
+        self.assertEqual(200, res.status_code)
+
+        self.assertEqual(0, len(res.json['packages']))
+
     def test_packages_filter_by_name(self):
-        """GET /catalog/packages with parameter "name" returns packages
+        """Test that packages are filtered by name
+
+        GET /catalog/packages with parameter "name" returns packages
         filtered by name.
         """
         self._set_policy_rules(
@@ -232,7 +408,9 @@ class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
                          res_pkgname2.json['packages'][0]['name'])
 
     def test_packages_filter_by_type(self):
-        """GET /catalog/packages with parameter "type" returns packages
+        """Test that packages are filtered by type
+
+        GET /catalog/packages with parameter "type" returns packages
         filtered by type.
         """
         self._set_policy_rules(
@@ -332,15 +510,15 @@ class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
             '/v1/catalog/packages/', params={'catalog': 'True'}))
         self.assertEqual(4, len(result['packages']))
 
-    def _test_package(self):
+    def _test_package(self, manifest='manifest.yaml'):
         package_dir = os.path.abspath(
             os.path.join(
                 __file__,
-                '../../../packages/test_packages/test.mpl.v1.app'
+                '../../../packages/test_packages/test.mpl.v1.app',
             )
         )
         pkg = load_utils.load_from_dir(
-            package_dir
+            package_dir, filename=manifest
         )
         package = {
             'fully_qualified_name': pkg.full_name,
@@ -360,6 +538,10 @@ class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
         }
         return pkg, package
 
+    def test_not_valid_logo(self):
+        self.assertRaises(exceptions.PackageLoadError,
+                          self._test_package, 'manifest_with_broken_logo.yaml')
+
     def test_load_package_with_supplier_info(self):
         self._set_policy_rules(
             {'get_package': '@'}
@@ -415,7 +597,7 @@ class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
         result = req.get_response(self.api)
 
         self.assertEqual(406, result.status_code)
-        self.assertTrue('Acceptable response can not be provided'
+        self.assertTrue(b'Acceptable response can not be provided'
                         in result.body)
 
     def test_get_ui_definition(self):
@@ -449,7 +631,7 @@ class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
         result = req.get_response(self.api)
 
         self.assertEqual(406, result.status_code)
-        self.assertTrue('Acceptable response can not be provided'
+        self.assertTrue(b'Acceptable response can not be provided'
                         in result.body)
 
     def test_get_logo(self):
@@ -484,7 +666,7 @@ class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
         result = req.get_response(self.api)
 
         self.assertEqual(406, result.status_code)
-        self.assertTrue('Acceptable response can not be provided'
+        self.assertTrue(b'Acceptable response can not be provided'
                         in result.body)
 
     def test_add_public_unauthorized(self):
@@ -506,7 +688,7 @@ class TestCatalogApi(test_base.ControllerTest, test_base.MuranoApiTestCase):
         file_obj.file = file_obj_str
         package_from_dir, _ = self._test_package()
 
-        body = '''\
+        body_fmt = '''\
 
 --BOUNDARY
 Content-Disposition: form-data; name="__metadata__"
@@ -518,6 +700,13 @@ Content-Disposition: form-data; name="ziparchive"; filename="file.zip"
 This is a fake zip archive
 --BOUNDARY--'''
 
+        def format_body(content):
+            content = jsonutils.dumps(content)
+            body = body_fmt.format(content)
+            if six.PY3:
+                body = body.encode('utf-8')
+            return body
+
         with mock.patch('murano.packages.load_utils.load_from_file') as lff:
             ctxmgr = mock.Mock()
             ctxmgr.__enter__ = mock.Mock(return_value=package_from_dir)
@@ -527,14 +716,14 @@ This is a fake zip archive
             # Uploading a non-public package
             req = self._post(
                 '/catalog/packages',
-                body.format(json.dumps({'is_public': False})),
+                format_body({'is_public': False}),
                 content_type='multipart/form-data; ; boundary=BOUNDARY',
             )
             res = req.get_response(self.api)
             self.assertEqual(200, res.status_code)
 
             self.is_admin = True
-            app_id = json.loads(res.body)['id']
+            app_id = jsonutils.loads(res.body)['id']
             req = self._delete('/catalog/packages/{0}'.format(app_id))
             res = req.get_response(self.api)
 
@@ -542,7 +731,7 @@ This is a fake zip archive
             # Uploading a public package fails
             req = self._post(
                 '/catalog/packages',
-                body.format(json.dumps({'is_public': True})),
+                format_body({'is_public': True}),
                 content_type='multipart/form-data; ; boundary=BOUNDARY',
             )
             res = req.get_response(self.api)
@@ -552,15 +741,14 @@ This is a fake zip archive
             self.is_admin = True
             req = self._post(
                 '/catalog/packages',
-                body.format(json.dumps({'is_public': True})),
+                format_body({'is_public': True}),
                 content_type='multipart/form-data; ; boundary=BOUNDARY',
             )
             res = req.get_response(self.api)
             self.assertEqual(200, res.status_code)
 
     def test_add_category(self):
-        """Check that category added successfully
-        """
+        """Check that category added successfully"""
 
         self._set_policy_rules({'add_category': '@'})
         self.expect_policy_check('add_category')
@@ -576,16 +764,15 @@ This is a fake zip archive
         }
 
         body = {'name': 'new_category'}
-        req = self._post('/catalog/categories', json.dumps(body))
+        req = self._post('/catalog/categories', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        processed_result = json.loads(result.body)
+        processed_result = jsonutils.loads(result.body)
         self.assertIn('id', processed_result.keys())
         expected['id'] = processed_result['id']
-        self.assertDictEqual(expected, processed_result)
+        self.assertEqual(expected, processed_result)
 
     def test_delete_category(self):
-        """Check that category deleted successfully
-        """
+        """Check that category deleted successfully"""
 
         self._set_policy_rules({'delete_category': '@'})
         self.expect_policy_check('delete_category',
@@ -602,12 +789,11 @@ This is a fake zip archive
 
         req = self._delete('/catalog/categories/12345')
         processed_result = req.get_response(self.api)
-        self.assertEqual('', processed_result.body)
+        self.assertEqual(b'', processed_result.body)
         self.assertEqual(200, processed_result.status_code)
 
     def test_add_category_failed_for_non_admin(self):
-        """Check that non admin user couldn't add new category
-        """
+        """Check that non admin user couldn't add new category"""
 
         self._set_policy_rules({'add_category': 'role:context_admin'})
         self.is_admin = False
@@ -617,13 +803,15 @@ This is a fake zip archive
         timeutils.utcnow.override_time = fake_now
 
         body = {'name': 'new_category'}
-        req = self._post('/catalog/categories', json.dumps(body))
+        req = self._post('/catalog/categories', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(403, result.status_code)
 
     def test_add_long_category(self):
-        """Check that category, that contains more then 80 characters
-           fails to add
+        """Test that category name does not exceed 80 characters
+
+        Check that a category that contains more then 80 characters
+        fails to be added
         """
 
         self._set_policy_rules({'add_category': '@'})
@@ -633,7 +821,7 @@ This is a fake zip archive
         timeutils.utcnow.override_time = fake_now
 
         body = {'name': 'cat' * 80}
-        req = self._post('/catalog/categories', json.dumps(body))
+        req = self._post('/catalog/categories', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(400, result.status_code)
         result_message = result.text.replace('\n', '')
@@ -651,7 +839,7 @@ This is a fake zip archive
         req = self._get('/catalog/categories')
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
-        result_categories = json.loads(result.body)['categories']
+        result_categories = jsonutils.loads(result.body)['categories']
         self.assertEqual(2, len(result_categories))
         self.assertEqual(names, [c['name'] for c in result_categories])
 
@@ -660,7 +848,7 @@ This is a fake zip archive
         self.expect_policy_check('get_category')
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
-        result_categories = json.loads(result.body)['categories']
+        result_categories = jsonutils.loads(result.body)['categories']
         self.assertEqual(names, [c['name'] for c in result_categories])
 
         names.reverse()
@@ -670,7 +858,7 @@ This is a fake zip archive
         self.expect_policy_check('get_category')
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
-        result_categories = json.loads(result.body)['categories']
+        result_categories = jsonutils.loads(result.body)['categories']
         self.assertEqual(names, [c['name'] for c in result_categories])
 
     def test_list_category_negative(self):
