@@ -13,9 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import json
-
 from oslo_config import fixture as config_fixture
+from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 
 from murano.api.v1 import templates
@@ -41,7 +40,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
 
         req = self._get('/templates')
         result = req.get_response(self.api)
-        self.assertEqual({'templates': []}, json.loads(result.body))
+        self.assertEqual({'templates': []}, jsonutils.loads(result.body))
 
     def test_create_env_templates(self):
         """Create an template, test template.show()."""
@@ -59,15 +58,16 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
                     'id': 'env_template_id',
                     'is_public': False,
                     'name': 'mytemp',
+                    'description_text': 'description',
                     'version': 0,
                     'created': timeutils.isotime(fake_now)[:-1],
                     'updated': timeutils.isotime(fake_now)[:-1]}
 
-        body = {'name': 'mytemp'}
-        req = self._post('/templates', json.dumps(body))
+        body = {'name': 'mytemp', 'description_text': 'description'}
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
 
-        self.assertEqual(expected, json.loads(result.body))
+        self.assertEqual(expected, jsonutils.loads(result.body))
 
         # Reset the policy expectation
         self.expect_policy_check('list_env_templates')
@@ -75,7 +75,8 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         req = self._get('/templates')
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
-        self.assertEqual({'templates': [expected]}, json.loads(result.body))
+        self.assertEqual({'templates': [expected]},
+                         jsonutils.loads(result.body))
 
         expected['services'] = []
 
@@ -83,7 +84,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
                                  {'env_template_id': self.uuids[0]})
         req = self._get('/templates/%s' % self.uuids[0])
         result = req.get_response(self.api)
-        self.assertEqual(expected, json.loads(result.body))
+        self.assertEqual(expected, jsonutils.loads(result.body))
 
     def test_list_public_env_templates(self):
         """Create an template, test templates.public()."""
@@ -95,16 +96,17 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         self.expect_policy_check('create_env_template')
 
         body = {'name': 'mytemp2', 'is_public': True}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        self.assertTrue(json.loads(result.body)['is_public'])
+        self.assertTrue(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('list_env_templates')
         req = self._get('/templates', {'is_public': True})
 
         result = req.get_response(self.api)
-        self.assertEqual(1, len(json.loads(result.body)))
-        self.assertTrue(json.loads(result.body)['templates'][0]['is_public'])
+        data = jsonutils.loads(result.body)
+        self.assertEqual(1, len(data))
+        self.assertTrue(data['templates'][0]['is_public'])
 
     def test_clone_env_templates(self):
         """Create an template, test templates.public()."""
@@ -115,18 +117,18 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
 
         self.expect_policy_check('create_env_template')
         body = {'name': 'mytemp2', 'is_public': True}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        env_template_id = json.loads(result.body)['id']
-        self.assertTrue(json.loads(result.body)['is_public'])
+        env_template_id = jsonutils.loads(result.body)['id']
+        self.assertTrue(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('clone_env_template')
         body = {'name': 'clone', 'is_public': False}
         req = self._post('/templates/%s/clone' % env_template_id,
-                         json.dumps(body))
+                         jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        self.assertFalse(json.loads(result.body)['is_public'])
-        self.assertEqual('clone', json.loads(result.body)['name'])
+        self.assertFalse(jsonutils.loads(result.body)['is_public'])
+        self.assertEqual('clone', jsonutils.loads(result.body)['name'])
 
     def test_clone_env_templates_private(self):
         """Create an template, test templates.public()."""
@@ -137,20 +139,22 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
 
         self.expect_policy_check('create_env_template')
         body = {'name': 'mytemp2', 'is_public': False}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        env_template_id = json.loads(result.body)['id']
-        self.assertFalse(json.loads(result.body)['is_public'])
+        env_template_id = jsonutils.loads(result.body)['id']
+        self.assertFalse(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('clone_env_template')
         body = {'name': 'clone', 'is_public': False}
         req = self._post('/templates/%s/clone' % env_template_id,
-                         json.dumps(body))
+                         jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(result.status_code, 403)
 
     def test_list_public_env_templates_default(self):
-        """Create an template, test list public with no
+        """Test listing public templates when there aren't any
+
+        Create a template; test list public with no
         public templates.
         """
         self._set_policy_rules(
@@ -160,18 +164,20 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
 
         self.expect_policy_check('create_env_template')
         body = {'name': 'mytemp'}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        self.assertFalse(json.loads(result.body)['is_public'])
+        self.assertFalse(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('list_env_templates')
         req = self._get('/templates', {'is_public': True})
         result = req.get_response(self.api)
 
-        self.assertFalse(0, len(json.loads(result.body)))
+        self.assertFalse(0, len(jsonutils.loads(result.body)))
 
     def test_list_private_env_templates(self):
-        """Create a public template and a private template,
+        """Test listing private templates
+
+        Create a public template and a private template;
         test list private templates.
         """
         self._set_policy_rules(
@@ -181,23 +187,25 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
 
         self.expect_policy_check('create_env_template')
         body = {'name': 'mytemp', 'is_public': False}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        self.assertFalse(json.loads(result.body)['is_public'])
+        self.assertFalse(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('create_env_template')
         body = {'name': 'mytemp1', 'is_public': True}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        self.assertTrue(json.loads(result.body)['is_public'])
+        self.assertTrue(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('list_env_templates')
         req = self._get('/templates', {'is_public': False})
         result = req.get_response(self.api)
-        self.assertEqual(1, len(json.loads(result.body)['templates']))
+        self.assertEqual(1, len(jsonutils.loads(result.body)['templates']))
 
     def test_list_env_templates(self):
-        """Create an template, test list public with no
+        """Test listing public templates when there aren't any
+
+        Create a template; test list public with no
         public templates.
         """
         self._set_policy_rules(
@@ -207,24 +215,26 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
 
         self.expect_policy_check('create_env_template')
         body = {'name': 'mytemp', 'is_public': False}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        self.assertFalse(json.loads(result.body)['is_public'])
+        self.assertFalse(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('create_env_template')
         body = {'name': 'mytemp1', 'is_public': True}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        self.assertTrue(json.loads(result.body)['is_public'])
+        self.assertTrue(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('list_env_templates')
         req = self._get('/templates')
         result = req.get_response(self.api)
 
-        self.assertEqual(2, len(json.loads(result.body)['templates']))
+        self.assertEqual(2, len(jsonutils.loads(result.body)['templates']))
 
     def test_list_env_templates_with_different_tenant(self):
-        """Create two template in two different tenants,
+        """Test listing public template from another tenant
+
+        Create two template in two different tenants;
         test list public template from another tenant.
         """
         self._set_policy_rules(
@@ -234,22 +244,23 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
 
         self.expect_policy_check('create_env_template')
         body = {'name': 'mytemp', 'is_public': False}
-        req = self._post('/templates', json.dumps(body), tenant='first_tenant')
+        req = self._post('/templates', jsonutils.dump_as_bytes(body),
+                         tenant='first_tenant')
         result = req.get_response(self.api)
-        self.assertFalse(json.loads(result.body)['is_public'])
+        self.assertFalse(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('create_env_template')
         body = {'name': 'mytemp1', 'is_public': True}
-        req = self._post('/templates', json.dumps(body),
+        req = self._post('/templates', jsonutils.dump_as_bytes(body),
                          tenant='second_tenant')
         result = req.get_response(self.api)
-        self.assertTrue(json.loads(result.body)['is_public'])
+        self.assertTrue(jsonutils.loads(result.body)['is_public'])
 
         self.expect_policy_check('list_env_templates')
         req = self._get('/templates', tenant='first_tenant')
         result = req.get_response(self.api)
 
-        self.assertEqual(2, len(json.loads(result.body)['templates']))
+        self.assertEqual(2, len(jsonutils.loads(result.body)['templates']))
 
     def test_illegal_template_name_create(self):
         """Check that an illegal temp name results in an HTTPClientError."""
@@ -261,7 +272,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         self.expect_policy_check('create_env_template')
 
         body = {'name': '  '}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(400, result.status_code)
 
@@ -275,7 +286,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         self.expect_policy_check('create_env_template')
 
         body = {'name': 'a' * 256}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(400, result.status_code)
         result_msg = result.text.replace('\n', '')
@@ -291,14 +302,12 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         self.expect_policy_check('create_env_template')
 
         body = {'invalid': 'test'}
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(400, result.status_code)
 
     def test_missing_env_template(self):
-        """Check that a missing environment template
-        results in an HTTPNotFound.
-        """
+        """Check that a missing env template results in an HTTPNotFound."""
         self._set_policy_rules(
             {'show_env_template': '@'}
         )
@@ -329,6 +338,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
             created=fake_now,
             updated=fake_now,
             tenant_id=self.tenant,
+            description_text='',
             description={
                 'name': 'my-temp',
                 '?': {'id': '12345'}
@@ -348,7 +358,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         body = {
             'name': 'renamed_temp'
         }
-        req = self._put('/templates/12345', json.dumps(body))
+        req = self._put('/templates/12345', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
 
@@ -361,7 +371,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         expected['created'] = timeutils.isotime(expected['created'])[:-1]
         expected['updated'] = timeutils.isotime(expected['updated'])[:-1]
 
-        self.assertEqual(expected, json.loads(result.body))
+        self.assertEqual(expected, jsonutils.loads(result.body))
 
     def test_delete_env_templates(self):
         """Test that environment deletion results in the correct rpc call."""
@@ -392,7 +402,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         result = req.get_response(self.api)
 
         # Should this be expected behavior?
-        self.assertEqual('', result.body)
+        self.assertEqual(b'', result.body)
         self.assertEqual(200, result.status_code)
 
     def test_create_env_templates_with_applications(self):
@@ -410,6 +420,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
                     'id': self.uuids[0],
                     'is_public': False,
                     'name': 'env_template_name',
+                    'description_text': '',
                     'version': 0,
                     'created': timeutils.isotime(fake_now)[:-1],
                     'updated': timeutils.isotime(fake_now)[:-1]}
@@ -460,9 +471,9 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
             ]
         }
 
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
-        self.assertEqual(expected, json.loads(result.body))
+        self.assertEqual(expected, jsonutils.loads(result.body))
 
         # Reset the policy expectation
         self.expect_policy_check('list_env_templates')
@@ -471,7 +482,8 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         result = req.get_response(self.api)
         del expected['services']
         self.assertEqual(200, result.status_code)
-        self.assertEqual({'templates': [expected]}, json.loads(result.body))
+        self.assertEqual({'templates': [expected]},
+                         jsonutils.loads(result.body))
 
         # Reset the policy expectation
         self.expect_policy_check('show_env_template',
@@ -479,7 +491,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         expected['services'] = services
         req = self._get('/templates/%s' % self.uuids[0])
         result = req.get_response(self.api)
-        self.assertEqual(expected, json.loads(result.body))
+        self.assertEqual(expected, jsonutils.loads(result.body))
 
     def test_add_application_to_template(self):
         """Create an template, test template.show()."""
@@ -516,21 +528,21 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
             "name": "template_name",
         }
 
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
 
         body = services
         req = self._post('/templates/%s/services' % self.uuids[0],
-                         json.dumps(body))
+                         jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
 
         self.assertEqual(200, result.status_code)
-        self.assertEqual(services, json.loads(result.body))
+        self.assertEqual(services, jsonutils.loads(result.body))
         req = self._get('/templates/%s/services' % self.uuids[0])
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
-        self.assertEqual(1, len(json.loads(result.body)))
+        self.assertEqual(1, len(jsonutils.loads(result.body)))
 
         service_no_instance = [
             {
@@ -545,14 +557,14 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         ]
 
         req = self._post('/templates/%s/services' % self.uuids[0],
-                         json.dumps(service_no_instance))
+                         jsonutils.dump_as_bytes(service_no_instance))
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
 
         req = self._get('/templates/%s/services' % self.uuids[0])
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
-        self.assertEqual(2, len(json.loads(result.body)))
+        self.assertEqual(2, len(jsonutils.loads(result.body)))
 
     def test_delete_application_in_template(self):
         """Create an template, test template.show()."""
@@ -579,14 +591,15 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
             ]
         }
 
-        req = self._post('/templates', json.dumps(body))
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
+        self.assertEqual(1, len(jsonutils.loads(result.body)['services']))
 
         req = self._get('/templates/%s/services' % self.uuids[0])
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
-        self.assertEqual(1, len(json.loads(result.body)))
+        self.assertEqual(1, len(jsonutils.loads(result.body)))
 
         service_id = '54cea43d-5970-4c73-b9ac-fea656f3c722'
         req = self._get('/templates/' + self.uuids[0] +
@@ -597,7 +610,9 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         req = self._delete('/templates/' + self.uuids[0] +
                            '/services/' + service_id)
         result = req.get_response(self.api)
+
         self.assertEqual(200, result.status_code)
+        self.assertEqual(0, len(jsonutils.loads(result.body)['services']))
 
         req = self._get('/templates/' + self.uuids[0] +
                         '/services/' + service_id)
@@ -621,16 +636,18 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         self.expect_policy_check('create_environment',
                                  {'env_template_id': self.uuids[0]})
         req = self._post('/templates/%s/create-environment' %
-                         self.uuids[0], json.dumps(body_env))
+                         self.uuids[0], jsonutils.dump_as_bytes(body_env))
         session_result = req.get_response(self.api)
         self.assertEqual(200, session_result.status_code)
         self.assertIsNotNone(session_result)
-        body_returned = json.loads(session_result.body)
+        body_returned = jsonutils.loads(session_result.body)
         self.assertEqual(self.uuids[4], body_returned['session_id'])
         self.assertEqual(self.uuids[3], body_returned['environment_id'])
 
     def test_create_env_with_template_no_services(self):
-        """Test that environment is created and session with template
+        """Test env and session creation without services
+
+        Test that environment is created and session with template
         without services.
         """
         self.fixture = self.useFixture(config_fixture.Config())
@@ -646,30 +663,87 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         body = {'name': 'my_template'}
 
         req = self._post('/templates/%s/create-environment' %
-                         self.uuids[0], json.dumps(body))
+                         self.uuids[0], jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertIsNotNone(result)
         self.assertEqual(200, result.status_code)
-        body_returned = json.loads(result.body)
+        body_returned = jsonutils.loads(result.body)
         self.assertEqual(self.uuids[4], body_returned['session_id'])
         self.assertEqual(self.uuids[3], body_returned['environment_id'])
 
+    def test_update_service_in_template(self):
+        """Test the service is updated in the environment template"""
+        self.fixture = self.useFixture(config_fixture.Config())
+        self.fixture.conf(args=[])
+        self._set_policy_rules(
+            {'create_env_template': '@',
+             'update_service_env_template': '@'}
+        )
+        updated_env = "UPDATED_ENV"
+        env_template = self._create_env_template_services()
+        self.expect_policy_check('update_service_env_template')
+        env_template["name"] = updated_env
+
+        req = self._put('/templates/{0}/services/{1}'.
+                        format(self.uuids[0], "service_id"),
+                        jsonutils.dump_as_bytes(env_template))
+        result = req.get_response(self.api)
+        self.assertIsNotNone(result)
+        self.assertEqual(200, result.status_code)
+        body_returned = jsonutils.loads(result.body)
+        self.assertEqual(updated_env, body_returned['name'])
+
     def test_mallformed_env_body(self):
         """Check that an illegal temp name results in an HTTPClientError."""
         self._set_policy_rules(
             {'create_env_template': '@',
              'create_environment': '@'}
         )
-        self. _create_env_template_no_service()
+        self._create_env_template_no_service()
 
         self.expect_policy_check('create_environment',
                                  {'env_template_id': self.uuids[0]})
         body = {'invalid': 'test'}
         req = self._post('/templates/%s/create-environment' %
-                         self.uuids[0], json.dumps(body))
+                         self.uuids[0], jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(400, result.status_code)
 
+    def test_delete_notexisting_service(self):
+        """Check deleting a not existing service, return a 404 error."""
+        self._set_policy_rules(
+            {'create_env_template': '@',
+             'delete_env_application': '@'}
+        )
+        self.expect_policy_check('create_env_template')
+
+        fake_now = timeutils.utcnow()
+        timeutils.utcnow.override_time = fake_now
+
+        body = {
+            "name": "mytemplate",
+            "services": [
+                {
+                    "name": "tomcat",
+                    "port": "8080",
+                    "?": {
+                        "type": "io.murano.apps.apache.Tomcat",
+                        "id": "ID1"
+                    }
+                }
+            ]
+        }
+
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
+        result = req.get_response(self.api)
+        self.assertEqual(200, result.status_code)
+        self.assertEqual(1, len(jsonutils.loads(result.body)['services']))
+
+        req = self._delete('/templates/{0}/services/{1}'.format(self.uuids[0],
+                                                                "NO_EXIST"))
+        result = req.get_response(self.api)
+        self.assertEqual(404, result.status_code)
+
     def test_create_env_notexisting_templatebody(self):
         """Check that an illegal temp name results in an HTTPClientError."""
         self._set_policy_rules(
@@ -681,7 +755,7 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
 
         body = {'name': 'test'}
         req = self._post('/templates/%s/create-environment'
-                         % env_template_id, json.dumps(body))
+                         % env_template_id, jsonutils.dump_as_bytes(body))
         result = req.get_response(self.api)
         self.assertEqual(404, result.status_code)
 
@@ -690,7 +764,8 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
         fake_now = timeutils.utcnow()
         timeutils.utcnow.override_time = fake_now
 
-        req = self._post('/templates', json.dumps({'name': 'name'}))
+        req = self._post('/templates',
+                         jsonutils.dump_as_bytes({'name': 'name'}))
         result = req.get_response(self.api)
         self.assertEqual(200, result.status_code)
 
@@ -720,11 +795,12 @@ class TestEnvTemplateApi(tb.ControllerTest, tb.MuranoApiTestCase):
                     "port": "8080",
                     "?": {
                         "type": "io.murano.apps.apache.Tomcat",
-                        "id": "54cea43d-5970-4c73-b9ac-fea656f3c722"
+                        "id": "service_id"
                     }
                 }
             ]
         }
 
-        req = self._post('/templates', json.dumps(body))
-        req.get_response(self.api)
+        req = self._post('/templates', jsonutils.dump_as_bytes(body))
+        result = req.get_response(self.api)
+        return result.json
