@@ -40,7 +40,7 @@ class CatalogDBTestCase(base.MuranoWithDBTestCase):
 
     def _stub_package(self, **kwargs):
         base = {
-            'archive': "archive blob here",
+            'archive': b"archive blob here",
             'fully_qualified_name': 'com.example.package',
             'type': 'class',
             'author': 'OpenStack',
@@ -49,7 +49,7 @@ class CatalogDBTestCase(base.MuranoWithDBTestCase):
             'description': 'some text',
             'is_public': False,
             'tags': ['tag1', 'tag2'],
-            'logo': "logo blob here",
+            'logo': b"logo blob here",
             'ui_definition': '{}',
         }
         base.update(**kwargs)
@@ -109,7 +109,9 @@ class CatalogDBTestCase(base.MuranoWithDBTestCase):
         self.assertEqual(pkg_a_id + pkg_z_id, [r.id for r in res])
 
     def test_pagination_backwards(self):
-        """Creates 10 packages with unique names and iterates backwards,
+        """Tests backwards pagination
+
+        Creates 10 packages with unique names and iterates backwards,
         checking that package order is correct.
         """
         pkgs = []
@@ -149,7 +151,9 @@ class CatalogDBTestCase(base.MuranoWithDBTestCase):
         self.assertEqual(0, len(res))
 
     def test_pagination(self):
-        """Creates 10 packages with unique names and iterates through them,
+        """Tests that package order is correct
+
+        Creates 10 packages with unique names and iterates through them,
         checking that package order is correct.
         """
 
@@ -182,8 +186,10 @@ class CatalogDBTestCase(base.MuranoWithDBTestCase):
         self.assertEqual(0, len(res))
 
     def test_pagination_loops_through_names(self):
-        """Creates 10 packages with the same display name and iterates through them,
-        checking that package are not skipped.
+        """Tests that packages with same display name are not skipped
+
+        Creates 10 packages with the same display name and iterates
+        through them, checking that package are not skipped.
         """
 
         for dummy in range(10):
@@ -374,7 +380,7 @@ class CatalogDBTestCase(base.MuranoWithDBTestCase):
         self.assertEqual(2, len(res))
 
         for cat in res:
-            self.assertTrue(cat.id is not None)
+            self.assertIsNotNone(cat.id)
             self.assertTrue(cat.name.startswith('cat'))
 
     def test_package_upload(self):
