@@ -77,6 +77,15 @@ class TestPackageLoader(package_loader.MuranoPackageLoader):
         else:
             raise KeyError(class_name)
 
+    def export_fixation_table(self):
+        return {}
+
+    def import_fixation_table(self, fixations):
+        pass
+
+    def compact_fixation_table(self):
+        pass
+
     def _build_index(self, directory):
         yamls = [
             os.path.join(dirpath, f)
@@ -100,7 +109,7 @@ class TestPackageLoader(package_loader.MuranoPackageLoader):
             for name, method in six.iteritems(data.get('Methods') or data.get(
                     'Workflow') or {}):
                 if name.startswith('test'):
-                    method['Usage'] = 'Action'
+                    method['Scope'] = 'Public'
 
             ns = namespace_resolver.NamespaceResolver(last_ns)
             class_name = ns.resolve_name(data['Name'])
