@@ -39,9 +39,16 @@ class MuranoTempestPlugin(plugins.TempestPlugin):
         config.register_opt_group(
             conf, config_application_catalog.service_broker_group,
             config_application_catalog.ServiceBrokerGroup)
+        config.register_opt_group(
+            conf, config_application_catalog.artifacts_group,
+            config_application_catalog.ArtifactsGroup)
 
     def get_opt_lists(self):
         return [(config_application_catalog.application_catalog_group.name,
                  config_application_catalog.ApplicationCatalogGroup),
                 (config_application_catalog.service_broker_group.name,
-                 config_application_catalog.ServiceBrokerGroup)]
+                 config_application_catalog.ServiceBrokerGroup),
+                ('service_available',
+                 config_application_catalog.ServiceAvailableGroup),
+                (config_application_catalog.artifacts_group.name,
+                 config_application_catalog.ArtifactsGroup)]
